export * from '../2.9/type';
import * as ts from 'typescript';
export declare function isTupleType(type: ts.Type): type is ts.TupleType;
export declare function isTupleTypeReference(type: ts.Type): type is ts.TypeReference & {
    target: ts.TupleType;
};
