export * from '../2.9/node';
import * as ts from 'typescript';
export declare function isOptionalTypeNode(node: ts.Node): node is ts.OptionalTypeNode;
export declare function isRestTypeNode(node: ts.Node): node is ts.RestTypeNode;
export declare function isSyntheticExpression(node: ts.Node): node is ts.SyntheticExpression;
